;;
;; $Id: test.scm,v 1.2 2000/01/25 12:45:51 shiro Exp $
;;

(load "pserializer")

(define (test num proc)
  (format #t "test ~s..." num)
  (if (catch
       (if (proc)
           (format #t "ok~%")
           (format #t "failed~%")))
      (format #t "failed by error~%")))

(define (topological-equal? a b)
  (let ((ctx (cons '() '()))
        (idx 0))
    (define (add-to-context a b)
      (set-car! ctx (cons (cons a idx) (car ctx)))
      (set-cdr! ctx (cons (cons b idx) (cdr ctx)))
      (set! idx (+ idx 1)))
    (define (get-object-id-a a)
      (let ((p (assq a (car ctx))))
        (and p (cdr p))))
    (define (get-object-id-b b)
      (let ((p (assq b (cdr ctx))))
        (and p (cdr p))))
    
    (define (tequal? a b)
      (cond ((or (boolean? a) (char? a) (number? a))
             (eqv? a b))
            ((eq? a '()) (eq? b '()))
            (else
             (let ((a-id (get-object-id-a a)))
               (if a-id                 ;object a has been appeared
                   (let ((b-id (get-object-id-b b)))
                     (eqv? a-id b-id))
                   (begin
                     (add-to-context a b)
                     (cond
                      ((pair? a) (and (pair? b)
                                      (tequal? (car a) (car b))
                                      (tequal? (cdr a) (cdr b))))
                      ((string? a)
                       (string=? a b))
                      ((symbol? a)
                       (and (symbol? b)
                            (string=? (symbol->string a)
                                      (symbol->string b))))
                      ((vector? a)
                       (and (vector? b)
                            (let loop ((la (vector->list a))
                                       (lb (vector->list b)))
                              (cond ((null? la)
                                     (null? lb))
                                    ((null? lb) #f)
                                    ((tequal? (car la) (car lb))
                                     (loop (cdr la) (cdr lb)))
                                    (else #f)))))
                      (else
                       (equal? a b))    ; is it correct?
                      )))
               ))
            )
      )

    (tequal? a b)
    ))

(define (objects->string . objs)
  (call-with-output-string
   (lambda (port)
     (call-with-output-serializer
      port
      (lambda (out)
        (for-each (lambda (item) (write-to-output-serializer item out))
                  objs))))))

(define (string->objects str)
  (call-with-input-string
   str
   (lambda (port)
     (call-with-input-serializer
      port
      (lambda (in)
        (let loop ((elt (read-from-input-serializer in))
                   (data '()))
          (if (eof-object? elt)
              (reverse data)
              (loop (read-from-input-serializer in)
                    (cons elt data)))))))))

;; test data

(define *primitive-types*
   '(1 -1                               ; smallint
     999999999999999 -888888888888888   ; bignum
     3.14178 5.0e33                     ; flonum
     #f #t                              ; boolean
     #\nul #\return #\A                 ; char
     ()                                 ; emptylist
     "string" ""                        ; string
     x y |Z|                            ; symbol
     (1 2 . 3)                          ; list
     #(a b c)                           ; vector
     )
   )

(define *shared-substructure*           ; don't print this!
  (let* ((str "shared string")
         (vec '#(shared vector))
         (circ (list 1 2 3 4))
         (circt (list-tail circ 3)))
    (set-cdr! circt circ)               ;make circ circular
    (list* str vec str vec circ)))

;; pserializer test

;; p1 : primitive types

(test 'p1
      (lambda ()
        (let* ((data *primitive-types*)
               (serialized (apply objects->string data))
               (retrieved (string->objects serialized))
               )
          ;;(display serialized)(newline)
          (equal? data retrieved))))

;; p2 : structure with shared/circular component

(test 'p2
      (lambda ()
        (let* ((data *shared-substructure*)
               (serialized (objects->string data))
               (retrieved (car (string->objects serialized)))
               )
          ;;(display serialized)(newline)
          (topological-equal? data retrieved))))

(exit 0)                                ;for stk
